#region Using declarations
using System;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// The ZigZag indicator shows trend lines filtering out changes below a defined level. 
    /// </summary>
    [Description("The ZigZag indicator shows trend lines filtering out changes below a defined level. ")]
    public class anaZigZag : Indicator
    {
        #region Variables
			private double			currentHigh			= 0.0;
			private double			currentLow			= 0.0;
			private double			normalizedRange		= 0.0;
			private double			rangeFactor			= 5.0;
			private double			tickSize			= 0.0;
			private double 			minimumDeviation	= 0.0;
			private int				count				= 0;
			private int				lastHighIdx			= -1;
			private int				lastLowIdx			= -1;
			private bool			upTrend				= true; 
			private bool			firstHighIdx		= false;
			private bool 			firstLowIdx			= false;
			private bool			addHigh				= false;
			private bool			updateHigh			= false;
			private bool			addLow				= false;
			private bool			updateLow			= false;
			private int 			plotWidth 			= 2;
			private DashStyle		dashStyle			= DashStyle.Solid;
			private Color			plotColor			= Color.CornflowerBlue;
 			private DataSeries		zigZagHigh; 
			private DataSeries		zigZagLow; 
       #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
        	Add(new Plot(new Pen(Color.Gray, 1), PlotStyle.Line, "ZigZag"));

			zigZagHigh	= new DataSeries(this); 
			zigZagLow	= new DataSeries(this); 
			
			MaximumBarsLookBack = MaximumBarsLookBack.Infinite;
            PriceTypeSupported	= false;
			PlotsConfigurable	= false;
			DisplayInDataBox	= false;
            Overlay				= true;
			PaintPriceMarkers	= false;
		}

        protected override void OnStartUp()
        {
			tickSize = Bars.Instrument.MasterInstrument.TickSize;
			upTrend = true; // arbitrary, just to start somwhere
			firstHighIdx = false;
			firstLowIdx = false;
			addHigh	= false;
			updateHigh = false;
			addLow = false;
			updateLow = false;
			lastHighIdx = -1;
			lastLowIdx = -1;
			Plots[0].Pen.Width = plotWidth;
			Plots[0].Pen.DashStyle = dashStyle;
			Plots[0].Pen.Color = plotColor;
		}	
	    
		/// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < 2) 
				return;
			if (FirstTickOfBar)
			{
				double trueRange = Math.Max(High[1],Close[2]) - Math.Min(Low[1],Close[2]);
				if(CurrentBar == 2)
					normalizedRange = trueRange;
				else
					normalizedRange =  normalizedRange*(double)(CurrentBar-1)/(double)CurrentBar + trueRange/CurrentBar; 
				minimumDeviation = rangeFactor * normalizedRange;
				if (addHigh || updateHigh)
					firstHighIdx = true;
				if (addLow || updateLow)
					firstLowIdx = true;
			}
			updateHigh = upTrend && High[0] > currentHigh;
			updateLow = !upTrend && Low[0] < currentLow;
			addHigh = !upTrend && !updateLow && High[0] > currentLow + minimumDeviation;
			addLow = upTrend && !updateHigh && Low[0] < currentHigh - minimumDeviation;
			
			if (addHigh)
			{
				upTrend = true;
				zigZagHigh.Set(High[0]);
				currentHigh = High[0];
				lastHighIdx = CurrentBar;
			}
			else if (updateHigh)
			{
				zigZagHigh.Set(High[0]);
				if (firstHighIdx)
					zigZagHigh.Set(CurrentBar - lastHighIdx, 0.0); 
				currentHigh = High[0];
				lastHighIdx = CurrentBar;
			}
			else
				zigZagHigh.Set(0.0);
			
			if (addLow)
			{
				upTrend = false;
				zigZagLow.Set(Low[0]);
				currentLow = Low[0];
				lastLowIdx = CurrentBar;
			}	
			else if (updateLow)	
			{
				zigZagLow.Set(Low[0]);
				if (firstLowIdx)
					zigZagLow.Set(CurrentBar - lastLowIdx, 0);
				currentLow = Low[0];
				lastLowIdx = CurrentBar;
			}
			else
				zigZagLow.Set(0.0);
			
			if(firstLowIdx && (addHigh || updateHigh))
			{
				count = CurrentBar - lastLowIdx;
				double lastLow = zigZagLow[count];
				double thisHigh = High[0];
				
				for (int i = 0; i <= count; i++)
					Value.Set(i, thisHigh * (count-i)/count + lastLow * i/count);   
			}	
			else if(firstHighIdx && (addLow || updateLow))
			{
				count = CurrentBar - lastHighIdx;
				double lastHigh = zigZagHigh[count];
				double thisLow = Low[0];
				
				for (int i = 0; i <= count; i++)
					Value.Set(i, thisLow * (count-i)/count + lastHigh * i/count);   
			}
			else
				Value.Reset();
        }

        #region Properties
        	[Description("Deviation measured in Average True Ranges")]
        [Category("Parameters")]
		[Gui.Design.DisplayName("ATR Multiple for Deviation")]
        public double RangeFactor
        {
            get { return rangeFactor; }
            set { rangeFactor = Math.Max(2.0, value); }
        }
		
		/// <summary>
		/// </summary>
		[Description("Line width for zigzag line")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Line Width")]
		public int PlotWidth
		{
			get { return plotWidth; }
			set { plotWidth = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for zigzag line")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style")]
		public DashStyle DashStyle
		{
			get { return dashStyle; }
			set { dashStyle = value; }
		} 
		
		/// <summary>
		/// </summary>
        [XmlIgnore()]		
		[Description("Select color for uptrend")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayName("Zig Zag Color")]
		public Color PlotColor
		{
			get { return plotColor; }
			set { plotColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string PlotColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(plotColor); }
			set { plotColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaZigZag[] cacheanaZigZag = null;

        private static anaZigZag checkanaZigZag = new anaZigZag();

        /// <summary>
        /// The ZigZag indicator shows trend lines filtering out changes below a defined level. 
        /// </summary>
        /// <returns></returns>
        public anaZigZag anaZigZag(double rangeFactor)
        {
            return anaZigZag(Input, rangeFactor);
        }

        /// <summary>
        /// The ZigZag indicator shows trend lines filtering out changes below a defined level. 
        /// </summary>
        /// <returns></returns>
        public anaZigZag anaZigZag(Data.IDataSeries input, double rangeFactor)
        {
            if (cacheanaZigZag != null)
                for (int idx = 0; idx < cacheanaZigZag.Length; idx++)
                    if (Math.Abs(cacheanaZigZag[idx].RangeFactor - rangeFactor) <= double.Epsilon && cacheanaZigZag[idx].EqualsInput(input))
                        return cacheanaZigZag[idx];

            lock (checkanaZigZag)
            {
                checkanaZigZag.RangeFactor = rangeFactor;
                rangeFactor = checkanaZigZag.RangeFactor;

                if (cacheanaZigZag != null)
                    for (int idx = 0; idx < cacheanaZigZag.Length; idx++)
                        if (Math.Abs(cacheanaZigZag[idx].RangeFactor - rangeFactor) <= double.Epsilon && cacheanaZigZag[idx].EqualsInput(input))
                            return cacheanaZigZag[idx];

                anaZigZag indicator = new anaZigZag();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.RangeFactor = rangeFactor;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaZigZag[] tmp = new anaZigZag[cacheanaZigZag == null ? 1 : cacheanaZigZag.Length + 1];
                if (cacheanaZigZag != null)
                    cacheanaZigZag.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaZigZag = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The ZigZag indicator shows trend lines filtering out changes below a defined level. 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaZigZag anaZigZag(double rangeFactor)
        {
            return _indicator.anaZigZag(Input, rangeFactor);
        }

        /// <summary>
        /// The ZigZag indicator shows trend lines filtering out changes below a defined level. 
        /// </summary>
        /// <returns></returns>
        public Indicator.anaZigZag anaZigZag(Data.IDataSeries input, double rangeFactor)
        {
            return _indicator.anaZigZag(input, rangeFactor);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The ZigZag indicator shows trend lines filtering out changes below a defined level. 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaZigZag anaZigZag(double rangeFactor)
        {
            return _indicator.anaZigZag(Input, rangeFactor);
        }

        /// <summary>
        /// The ZigZag indicator shows trend lines filtering out changes below a defined level. 
        /// </summary>
        /// <returns></returns>
        public Indicator.anaZigZag anaZigZag(Data.IDataSeries input, double rangeFactor)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaZigZag(input, rangeFactor);
        }
    }
}
#endregion
